@@ -1404,6 +1404,99 @@ namespace glz
       }
    };
 
+   template <is_expected T>
+   struct from<BEVE, T> final
+   {
+      template <auto Opts>
+      static void op(auto&& value, is_context auto&& ctx, auto&& it, auto end)
+      {
+         constexpr uint8_t object_header = tag::object; // string keys
+
+         if (invalid_end(ctx, it, end)) {
+            return;
+         }
+
+         auto parse_val = [&] {
+            if constexpr (not std::is_void_v<typename std::decay_t<T>::value_type>) {
+               if (value) {
+                  parse<BEVE>::op<Opts>(*value, ctx, it, end);
+               }
+               else {
+                  value.emplace();
+                  parse<BEVE>::op<Opts>(*value, ctx, it, end);
+               }
+            }
+            else {
+               value.emplace();
+            }
+         };
+
+         const auto tag = uint8_t(*it);
+         if ((tag & 0b111) == object_header) {
+            auto start = it;
+            ++it;
+
+            const auto n_keys = int_from_compressed(ctx, it, end);
+            if (bool(ctx.error)) [[unlikely]] {
+               return;
+            }
+
+            if (n_keys == 0) {
+               // empty object
+               if constexpr (std::is_void_v<typename std::decay_t<T>::value_type>) {
+                  value.emplace();
+               }
+               else {
+                  // rewind and parse as value (the value type might be an empty object)
+                  it = start;
+                  parse_val();
+               }
+            }
+            else if (n_keys == 1) {
+               // could be unexpected wrapper or a single-field object value
+               const auto key_len = int_from_compressed(ctx, it, end);
+               if (bool(ctx.error)) [[unlikely]] {
+                  return;
+               }
+
+               static constexpr sv unexpected_key = "unexpected";
+               if (key_len == unexpected_key.size() && uint64_t(end - it) >= key_len) {
+                  if (std::memcmp(it, unexpected_key.data(), key_len) == 0) {
+                     // this is an unexpected wrapper
+                     it += key_len;
+
+                     using error_type = typename std::decay_t<T>::error_type;
+                     if (!value) {
+                        parse<BEVE>::op<Opts>(value.error(), ctx, it, end);
+                     }
+                     else {
+                        std::decay_t<error_type> error{};
+                        parse<BEVE>::op<Opts>(error, ctx, it, end);
+                        if (bool(ctx.error)) [[unlikely]] {
+                           return;
+                        }
+                        value = glz::unexpected(std::move(error));
+                     }
+                     return;
+                  }
+               }
+               // not an unexpected wrapper, rewind and parse as value
+               it = start;
+               parse_val();
+            }
+            else {
+               // multiple keys, must be a value object
+               it = start;
+               parse_val();
+            }
+         }
+         else {
+            // not an object, parse as value directly
+            parse_val();
+         }
+      }
+   };
+
    template <nullable_t T>
       requires(std::is_array_v<T>)
    struct from<BEVE, T> final
@@ -1416,7 +1509,7 @@ namespace glz
    };
 
    template <nullable_t T>
-      requires(!std::is_array_v<T>)
+      requires(!std::is_array_v<T> && not is_expected<T>)
    struct from<BEVE, T> final
    {
       template <auto Opts>
