@@ -3,7 +3,6 @@
 
 #pragma once
 
-#include <format>
 #include <string_view>
 #include <type_traits>
 
@@ -12,6 +11,13 @@
 #include "glaze/json/write.hpp"
 #include "glaze/util/dump.hpp"
 
+// GLZ_USE_STD_FORMAT_FLOAT is defined in write_chars.hpp (included via write.hpp)
+#if GLZ_USE_STD_FORMAT_FLOAT
+#include <format>
+#else
+#include <cstdio>
+#endif
+
 namespace glz
 {
    // Compile-time string for use as template parameter
@@ -56,11 +62,12 @@ namespace glz
             }
          }
 
-         if constexpr (Opts.quoted_num) {
-            dump<'"'>(b, ix);
+         if constexpr (check_quoted_num(Opts)) {
+            dump('"', b, ix);
          }
 
-         constexpr auto fmt = std::format_string<V>{std::string_view{Fmt}};
+#if GLZ_USE_STD_FORMAT_FLOAT
+         constexpr auto fmt = std::format_string<V>{std::string_view { Fmt }};
 
          if constexpr (check_write_unchecked(Opts)) {
             // Caller guarantees buffer has enough space
@@ -89,9 +96,36 @@ namespace glz
             }
             ix += size;
          }
+#else
+         // snprintf fallback for platforms without std::format float support (e.g., iOS < 16.3)
+         constexpr auto printf_fmt = detail::to_printf_fmt(Fmt.data);
 
-         if constexpr (Opts.quoted_num) {
-            dump<'"'>(b, ix);
+         const auto start = reinterpret_cast<char*>(&b[ix]);
+         const auto available = check_write_unchecked(Opts) ? size_t(64) : (b.size() - ix);
+
+         const int len = std::snprintf(start, available, printf_fmt.data, static_cast<double>(wrapper.val));
+
+         if (len < 0) {
+            ctx.error = error_code::unexpected_end;
+            return;
+         }
+
+         if (static_cast<size_t>(len) >= available) {
+            if constexpr (resizable<B> && not check_write_unchecked(Opts)) {
+               b.resize(2 * (ix + static_cast<size_t>(len) + 1));
+               std::snprintf(reinterpret_cast<char*>(&b[ix]), static_cast<size_t>(len) + 1, printf_fmt.data,
+                             static_cast<double>(wrapper.val));
+            }
+            else {
+               ctx.error = error_code::unexpected_end;
+               return;
+            }
+         }
+         ix += static_cast<size_t>(len);
+#endif
+
+         if constexpr (check_quoted_num(Opts)) {
+            dump('"', b, ix);
          }
       }
    };
